@@ -1,8 +1,7 @@
 //! This example plays each oscillator type sequentially
-use web_audio_api::context::{AsBaseAudioContext, AudioContext};
-use web_audio_api::node::{
-    AudioNode, AudioScheduledSourceNode, OscillatorType, PeriodicWaveOptions,
-};
+use web_audio_api::context::{AudioContext, BaseAudioContext};
+use web_audio_api::node::{AudioNode, AudioScheduledSourceNode, OscillatorType};
+use web_audio_api::periodic_wave::PeriodicWaveOptions;
 
 fn main() {
     env_logger::init();
@@ -11,17 +10,17 @@ fn main() {
     let context = AudioContext::new(None);
 
     // Create an oscillator node with sine (default) type
-    let mut osc = context.create_oscillator();
+    let osc = context.create_oscillator();
 
     // Connect osc to the destination node which is the default output device
     osc.connect(&context.destination());
 
     // defined periodic wave characteristics
-    let options = Some(PeriodicWaveOptions {
+    let options = PeriodicWaveOptions {
         real: Some(vec![0., 0.5, 0.5]),
         imag: Some(vec![0., 0., 0.]),
-        disable_normalization: Some(false),
-    });
+        disable_normalization: false,
+    };
 
     // Create a custom periodic wave
     let periodic_wave = context.create_periodic_wave(options);
