@@ -1,4 +1,4 @@
-use crate::context::{AsBaseAudioContext, AudioContextRegistration};
+use crate::context::{AudioContextRegistration, BaseAudioContext};
 use crate::render::{AudioParamValues, AudioProcessor, AudioRenderQuantum};
 use crate::SampleRate;
 
@@ -7,7 +7,7 @@ use super::{
 };
 
 /// Representing the final audio destination and is what the user will ultimately hear.
-pub struct DestinationNode {
+pub struct AudioDestinationNode {
     pub(crate) registration: AudioContextRegistration,
     pub(crate) channel_count: usize,
 }
@@ -34,7 +34,7 @@ impl AudioProcessor for DestinationRenderer {
     }
 }
 
-impl AudioNode for DestinationNode {
+impl AudioNode for AudioDestinationNode {
     fn registration(&self) -> &AudioContextRegistration {
         &self.registration
     }
@@ -72,8 +72,8 @@ impl AudioNode for DestinationNode {
     }
 }
 
-impl DestinationNode {
-    pub fn new<C: AsBaseAudioContext>(context: &C, channel_count: usize) -> Self {
+impl AudioDestinationNode {
+    pub fn new<C: BaseAudioContext>(context: &C, channel_count: usize) -> Self {
         context.base().register(move |registration| {
             let node = Self {
                 registration,
