@@ -346,13 +346,16 @@ impl IirFilterRenderer {
 mod test {
     use float_eq::assert_float_eq;
     use realfft::num_traits::Zero;
-    use std::{cmp::min, fs::File};
+    use std::{
+        cmp::min,
+        fs::File,
+        io::{BufRead, BufReader},
+    };
 
     use crate::{
         context::{BaseAudioContext, OfflineAudioContext},
-        media::{MediaDecoder, MediaElement},
         node::{AudioNode, AudioScheduledSourceNode},
-        snapshot, SampleRate,
+        SampleRate,
     };
 
     use super::{ChannelConfigOptions, IIRFilterNode, IIRFilterOptions};
@@ -584,20 +587,17 @@ mod test {
     fn default_periodic_wave_rendering_should_match_snapshot() {
         // the snapshot data has been verified by fft to make sure that the frequency
         // response correspond to a HP filter with Fc 4000 Hz
-        let ref_filtered =
-            snapshot::read("./snapshots/white_hp.json").expect("Reading snapshot file failed");
+        let file = File::open("./snapshots/white_hp.json").expect("Reading snapshot file failed");
+        let reader = BufReader::new(file);
+        let ref_filtered: Vec<f32> = reader
+            .lines()
+            .map(|l| l.unwrap().parse::<f32>().unwrap())
+            .collect();
 
         let mut context = OfflineAudioContext::new(1, LENGTH, SampleRate(44_100));
 
-        // setup background music:
-        // read from local file
         let file = File::open("samples/white.ogg").unwrap();
-        // decode file to media stream
-        let stream = MediaDecoder::try_new(file).unwrap();
-        // wrap stream in MediaElement, so we can control it (loop, play/pause)
-        let media = MediaElement::new(stream);
-        // register as media element in the audio context
-        let background = context.create_media_element_source(media);
+        let audio_buffer = context.decode_audio_data_sync(file).unwrap();
 
         let feedforward = vec![
             0.019_618_022_238_052_212,
@@ -612,13 +612,17 @@ mod test {
             channel_config: ChannelConfigOptions::default(),
         };
         let iir = IIRFilterNode::new(&context, options);
-
-        background.connect(&iir);
         iir.connect(&context.destination());
 
-        background.start();
+        let src = context.create_buffer_source();
+        src.set_buffer(audio_buffer);
+        src.connect(&iir);
+        src.start();
+
         let output = context.start_rendering_sync();
 
+        // review the following, this should be fixed using an AudioBufferSourceNode
+
         // retrieve processed data by removing silence chunk
         // These silence slices are inserted inconsistently from an test execution to another
         // Without these processing the test would be brittle
@@ -630,11 +634,13 @@ mod test {
             .copied()
             .collect();
 
-        let ref_data_ch = ref_filtered.data;
-
-        let min_len = min(data_ch.len(), ref_data_ch.len());
+        let min_len = min(data_ch.len(), ref_filtered.len());
 
         // todo instable test
-        assert_float_eq!(data_ch[0..min_len], ref_data_ch[0..min_len], abs_all <= 1.0);
+        assert_float_eq!(
+            data_ch[0..min_len],
+            ref_filtered[0..min_len],
+            abs_all <= 1.0
+        );
     }
 }
