@@ -94,7 +94,6 @@ impl From<u32> for OscillatorType {
 /// # Usage
 ///
 /// ```no_run
-/// use std::fs::File;
 /// use web_audio_api::context::{BaseAudioContext, AudioContext};
 /// use web_audio_api::node::{AudioNode, AudioScheduledSourceNode};
 ///
@@ -129,12 +128,6 @@ pub struct OscillatorNode {
     sender: Sender<PeriodicWave>,
 }
 
-impl AudioScheduledSourceNode for OscillatorNode {
-    fn scheduler(&self) -> &Scheduler {
-        &self.scheduler
-    }
-}
-
 impl AudioNode for OscillatorNode {
     fn registration(&self) -> &AudioContextRegistration {
         &self.registration
@@ -155,6 +148,26 @@ impl AudioNode for OscillatorNode {
     }
 }
 
+impl AudioScheduledSourceNode for OscillatorNode {
+    fn start(&self) {
+        let when = self.registration.context().current_time();
+        self.start_at(when);
+    }
+
+    fn start_at(&self, when: f64) {
+        self.scheduler.start_at(when);
+    }
+
+    fn stop(&self) {
+        let when = self.registration.context().current_time();
+        self.stop_at(when);
+    }
+
+    fn stop_at(&self, when: f64) {
+        self.scheduler.stop_at(when);
+    }
+}
+
 impl OscillatorNode {
     /// Returns an `OscillatorNode`
     ///
@@ -388,7 +401,7 @@ impl AudioProcessor for OscillatorRenderer {
                 if current_time > start_time {
                     let phase_incr = computed_frequency as f64 / sample_rate;
                     let ratio = (current_time - start_time) / dt;
-                    self.phase = Self::unroll_phase(phase_incr as f64 * ratio);
+                    self.phase = Self::unroll_phase(phase_incr * ratio);
                 }
 
                 self.started = true;
