@@ -83,7 +83,7 @@ impl Default for WaveShaperOptions {
 /// ```no_run
 /// use std::fs::File;
 /// use web_audio_api::context::{BaseAudioContext, AudioContext};
-/// use web_audio_api::node::AudioNode;
+/// use web_audio_api::node::{AudioNode, AudioScheduledSourceNode};
 ///
 /// # use std::f32::consts::PI;
 /// # fn make_distortion_curve(size: usize) -> Vec<f32> {
@@ -401,33 +401,17 @@ impl WaveShaperRenderer {
         let channels_x2 = 1;
         let channels_x4 = 1;
 
-        let upsampler_x2 = FftFixedInOut::<f32>::new(
-            sample_rate as usize,
-            sample_rate as usize * 2,
-            256,
-            channels_x2,
-        );
+        let upsampler_x2 =
+            FftFixedInOut::<f32>::new(sample_rate, sample_rate * 2, 256, channels_x2);
 
-        let downsampler_x2 = FftFixedInOut::<f32>::new(
-            sample_rate as usize * 2,
-            sample_rate as usize,
-            128,
-            channels_x2,
-        );
+        let downsampler_x2 =
+            FftFixedInOut::<f32>::new(sample_rate * 2, sample_rate, 128, channels_x2);
 
-        let upsampler_x4 = FftFixedInOut::<f32>::new(
-            sample_rate as usize,
-            sample_rate as usize * 4,
-            512,
-            channels_x4,
-        );
+        let upsampler_x4 =
+            FftFixedInOut::<f32>::new(sample_rate, sample_rate * 4, 512, channels_x4);
 
-        let downsampler_x4 = FftFixedInOut::<f32>::new(
-            sample_rate as usize * 4,
-            sample_rate as usize,
-            128,
-            channels_x4,
-        );
+        let downsampler_x4 =
+            FftFixedInOut::<f32>::new(sample_rate * 4, sample_rate, 128, channels_x4);
 
         Self {
             sample_rate,
@@ -472,6 +456,7 @@ mod tests {
     use float_eq::assert_float_eq;
 
     use crate::context::OfflineAudioContext;
+    use crate::node::AudioScheduledSourceNode;
     use crate::SampleRate;
 
     use super::*;
